package convert

import (
	"git.chillcy.com/golang/chillcy/project/pc4/pkg/rpc/pb/pc4"
	model "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/product"
	"git.chillcy.com/golang/colorway/internal/rpc/internal/svc/cache"
	"sync"
)

type Product struct {
	cache *cache.Cache
}

func NewProduct(cache *cache.Cache) *Product {
	return &Product{cache: cache}
}

func (receiver *Product) Convert(designerProducts []*pc4.DesignerProduct) model.Products {
	var ps model.Products
	for _, designerProduct := range designerProducts {
		once := sync.Once{}
		p := model.Product{}
		styleNumber := designerProduct.GetDesignerId()
		for _, colorProduct := range designerProduct.GetColorProducts() {
			for _, productAndSkus := range colorProduct.GetProductAndSkusList() {
				product := productAndSkus.GetProduct()
				once.Do(func() {
					p.StyleNumber = styleNumber
					p.Title = model.ProductTitle{
						En: product.GetTitleEn(),
						Cn: product.GetTitleCn(),
					}
					p.Description = product.GetDetail().GetDescription()
					p.Season = product.GetSeason()
					p.Brand = model.ProductBrand{
						Id:   product.GetBrandId(),
						Name: product.GetBrandName(),
					}
					p.Care = product.GetDetail().GetCare()
					p.MadeIn = product.GetDetail().GetMadeIn()
					p.HsCode = product.GetDetail().GetHsCode()
					p.InfoSelect = product.GetId()
				})
			}
		}
		p.Colors = buildProductColors(designerProduct.GetColorProducts(), designerProduct.GetDesignerId())
		ps = append(ps, p)
	}
	return ps
}

func buildProductColors(colorProducts []*pc4.ColorProduct, styleNumber string) model.ProductColors {
	var pcs model.ProductColors
	for _, colorProduct := range colorProducts {
		pcs = append(pcs, buildProductColor(colorProduct, styleNumber))
	}
	return pcs
}

func buildProductColor(colorProduct *pc4.ColorProduct, styleNumber string) model.ProductColor {
	pc := model.ProductColor{}
	once := &sync.Once{}
	for _, productAndSkus := range colorProduct.GetProductAndSkusList() {
		product := productAndSkus.GetProduct()
		once.Do(func() {
			pc.ColorCode = colorProduct.GetColorNumber()
			pc.Id = product.GetColor().GetId()
			pc.Name = product.GetColor().GetName()
			pc.BasicId = product.GetColor().GetBasicColorId()
			pc.BasicName = product.GetColor().GetBasicColorName()
			pc.InfoSelect = product.GetId()
			pc.Images = product.GetColor().GetImages()
		})
		for _, sku := range productAndSkus.GetSkus() {
			pc.Skus = append(pc.Skus, model.ProductColorSku{
				Pc3SkuId:            sku.GetId(),
				Pc3ProductId:        product.GetId(),
				ColorSpuStyleNumber: styleNumber,
				ColorSpuColorCode:   colorProduct.GetColorNumber(),
				SourceId:            product.GetSourceId(),
				SiteId:              product.GetSiteId(),
				Size:                sku.GetRawSize(),
				Stock:               int(sku.GetStock()),
				Price:               sku.GetPriceDetail().GetNow(),
				Currency:            sku.GetPriceDetail().GetCurrency(),
			})
		}
	}
	return pc
}
